@@ -21,7 +21,10 @@ if ($user->requireReload() || ! $user->isLogin() || ! $lang->isManager()) {
   header('Location: index.php');
   return;
 }
-$lang->handleAdminRequest();
+if ($lang->handleAdminRequest()) {
+  header('Location: forManager.php');
+  return;
+}
 
 $o_smarty->assign('user', $user);
 $o_smarty->assign('lang', $lang);
