@@ -116,15 +116,17 @@ class UserAuthorize {
       fclose($fp);
     } else {
       // 桼ե뤬¸ߤʤ硢admin ѥɤɲä
-      $fp = fopen($_SESSION['users_file'], 'w');
-      if ($fp != false) {
-	flock($fp, LOCK_EX);
-	$line = "admin\t". crypt('admin'). "\n";
-	fwrite($fp, $line);
-	fclose($fp);
-	$_SESSION['login_name'] = 'admin';
-	$this->reload_acton = true;
-	$this->login_fail = false;
+      if (isset($_SESSION[''])) {
+	$fp = @fopen($_SESSION['users_file'], 'w');
+	if ($fp != false) {
+	  flock($fp, LOCK_EX);
+	  $line = "admin\t". crypt('admin'). "\n";
+	  fwrite($fp, $line);
+	  fclose($fp);
+	  $_SESSION['login_name'] = 'admin';
+	  $this->reload_acton = true;
+	  $this->login_fail = false;
+	}
       }
     }
     return ! $this->login_fail;
@@ -483,9 +485,9 @@ class LangUpload {
       $total_num = $exists[1][$chapter_no -1];
       $progress = (int)(100.0 * $exists_num / $total_num);
       $images = '&nbsp;';
-      $line = sprintf(" (%d/%d)</td><td width=255>%3d%% ",
-		      $exists_num, $total_num, $progress);
-      $ret .= preg_replace('/ /', '&nbsp;', $line);
+      $line = sprintf(" (%d/%d)</td><td width=255>", $exists_num, $total_num);
+      $add = sprintf("%3d%% ", $progress);
+      $ret .= $line. preg_replace('/ /', '&nbsp;', $add);
       $ret .=
 	sprintf('<img src=bar.png height=14 width=%d>',
 		2 * $progress);
@@ -499,8 +501,10 @@ class LangUpload {
     // ΤοĽɽ
     $progress = (int)(100.0 * $total_exists / $total_problems);
     $ret .= '<tr height=28><td>&nbsp;'. $total_tag;
-    $ret .= sprintf(' (%d/%d)</td><td width=255>&nbsp;%3d%% ',
-		    $total_exists, $total_problems, $progress);
+    $ret .= sprintf(' (%d/%d)</td><td width=255>',
+		    $total_exists, $total_problems);
+    $add = sprintf('%3d%% ', $progress);
+    $ret .= preg_replace('/ /', '&nbsp;', $add);
     $ret .= sprintf('<img src=bar.png height=16 width=%d>',
 		    2 * $progress);
     $ret .= '</td></tr>';
@@ -622,7 +626,7 @@ class LangUpload {
       } else {
 	$lines = @file($path);
 	if (is_array($lines)) {
-	  $out .= '<pre>'. join($lines). '</pre>';
+	  $out .= '<pre>'. htmlspecialchars(join($lines)). '</pre>';
 	} else {
 	  $out .= $no_program_message;
 	}
@@ -843,12 +847,12 @@ class LangUpload {
       // ̾ιʸå
       if (preg_match('/[^A-Za-z0-9_]/', $name)) {
 	// !!! [A-Za-z0-9_] ˤơȤɽ桼󼨤
-	return;
+	return true;
       }
       // ǥ쥯ȥκ
       // !!! Apache 桼äʬʬäʤΤǡѡߥå
       if (! @mkdir('data/'. $name, 0777)) {
-	return;
+	return true;
       }
       @chmod('data/'. $name, 0777);
 
@@ -860,6 +864,7 @@ class LangUpload {
 	fwrite($fp, $line);
 	fclose($fp);
       }
+      return true;
     }
 
     // ѥѹ
@@ -867,7 +872,7 @@ class LangUpload {
       // ɤǡ桼ʬΥѥɤΤѹơ񤭹
       if (! isset($_POST['change_user'])) {
 	print 'no change_user<br>';
-	return;
+	return true;
       }
       $change_user = $_POST['change_user'];
 
@@ -891,11 +896,40 @@ class LangUpload {
 	fwrite($fp, $replace_file);
 	fclose($fp);
       }
+      return true;
     }
 
     // 桼
     if (isset($_POST['del_user_submit'])) {
+      // ɤǡ桼ƽ񤭹
+      if (! isset($_POST['del_user'])) {
+	print 'no del_user<br>';
+	return true;
+      }
+      $del_user = $_POST['del_user'];
+
+      $replace_file = '';
+      $fp = fopen($_SESSION['users_file'], 'r');
+      if ($fp != false) {
+	flock($fp, LOCK_SH);
+	while (! feof($fp)) {
+	  $line = fgets($fp);
+	  $name = strtok($line, "\n\t ");
+	  if ($name != $del_user) {
+	    $replace_file .= $line;
+	  }
+	}
+	fclose($fp);
+      }
+      $fp = fopen($_SESSION['users_file'], 'w');
+      if ($fp != false) {
+	flock($fp, LOCK_EX);
+	fwrite($fp, $replace_file);
+	fclose($fp);
+      }
+      return true;
     }
+    return false;
   }
 
 
